/*
   Der Controller regelt den Spielablauf
*/
class Controller                                 //Kontrolliert den Spielablauf
{
   public:
     Controller(TForm1*);                        //Konstruktor
     Spieler* getSpieler(int);                   //Liefert einen Spieler
     TForm1* getGUI();                            //GUI zurckgeben
     SpielfeldModel* getModel();                 //Liefert das Model (Speicher aller Felder)
     void reset();                               //Spiel zurcksetzten (Neustart)
     void setPlayerType(int id, AnsiString type); //Typ setzen
     void refreshStatus();                       //Aktualisiert die Statusanzeige
     void refreshTimer();                        //Aktualisiert die Zeitanzeige
     void resetTimer();                          //Uhr auf 0 setzen

   private:
     TForm1 *gui;                                //Die GUI
     Spieler *spieler[2];                        //Das Array der Spieler
     int counter;                                //Zhlt die Spielzge
     SpielfeldModel* model;                      //Model
     unsigned int timer_min;                     //Minutenanzeige
     unsigned int timer_sec;                     //Sekundenanzeige
     bool finished;                              //Spiel beendet?
     bool isCombination(Spieler*, int, int, int,
                             int, unsigned int); //Kombination?
     void __fastcall onClick(TObject *Sender);   //Behandelt einen Klick
     void __fastcall adjust(TObject *Sender);    //Passt das Spielfeld an
     void setFinished(bool);                     //Spiel beenden / starten
     bool isFinished();                          //Spiel beendet?
     void zugPC();                               //Zug des PC-Spielers
     void pcEasy();                              //PC-Schwierigkeit Einfach
     void pcDifficult();                         //PC-Schwierigkeit Schwer
     bool isUnentschieden();                     //Ist das Spiel unentschieden?
     int getCurrentPlayerID();                   //Die ID des aktuellen Spielers zurckgeben
     Spieler* getCurrentPlayer();                //Den aktuellen Spieler zurckgeben
     int getNextPlayerID();                      //Die nchste Spieler-ID
     Spieler* getNextPlayer();                   //Nchstes Spieler-Objekt
     void searchWinner();                        //Sucht gewinner / Unentschieden
     void incrementCounter();                    //1 Spielzug mehr
     void decrementCounter();                    //1 Spielzug weniger
     int getCounter();                           //gibt Anzahl der Spielzge zurck
     void resetCounter();                        //Setzt die Spielzge auf 0 zurck
     Feld* findCombinations(Spieler*, unsigned int);      //Findet Kombinationen
     Spieler* checkField();                      //Ermittelt einen Gewinner
};

//Konstruktor initialisiert Variablen und erstellt Objekte
Controller::Controller(TForm1* Form1)
{
   this->gui = Form1;                   //Das erste Formular als GUI verwenden

   //Die Breite der GUI fetlegen
   this->gui->Width = (CONST__anzX+1)* (CONST__breite_label + CONST__abstand) + CONST__abstand;

   //Die Hhe der GUI festlegen
   this->gui->Height = (CONST__anzY+1)* (CONST__hoehe_label + CONST__abstand) + CONST__abstand + 10;

   //Eventhandler des Feldcontainers beim Resize (auch beim Hinzufgen von Elementen)
   this->gui->panel_feldcontainer->OnResize = adjust;

   //Eventhandler der GUI beim Resitze = OnResize von Feldcontainer
   this->gui->OnResize = this->gui->panel_feldcontainer->OnResize;

   //Das Model, das das Spielfeld enthlt, wird erstellt.
   this->model = new SpielfeldModel(
                                  this->gui, this->gui->panel_feldcontainer,
                                  CONST__breite_label,
                                  CONST__hoehe_label,
                                  (CONST__hoehe_label + CONST__abstand),
                                  (CONST__breite_label + CONST__abstand)
                                 );

   //Jetzt muss noch jedem Feld die onClick-Methode zugewiesen werden.
   for(int y = 0; y < CONST__anzY; y++)
   {
     for(int x = 0; x < CONST__anzX; x++)
     {
       model->getFeld(x,y)->OnClick = onClick;
     }
   }

   this->spieler[0] = new Mensch;                             //Ersten Spieler erzeugen
   this->spieler[0]->setColor(CONST__default_spieler1_color); //Farbe setzen

   this->spieler[1] = new Computer(2);                        //Zweiten Spieler erzeugen
   this->spieler[1]->setColor(CONST__default_spieler2_color); //Farbe setzen

   this->counter = 0;                   //Es wurde noch nicht gespielt
   this->finished = false;              //Das Spiel ist noch nicht beendet
   this->timer_min = 0;                 //Timer zurcksetzen
   this->timer_sec = 0;                 //Timer zurcksetzen
   this->refreshStatus();               //Statusanzeige aktualisieren (bzw. initialisieren)
}

// Diese Methode regelt den Spielablauf. Mit dem ersten Klick eines Spielers beginnt das Spiel.
// Ist der zweite Spieler der PC, wird automatisch die entsprechende Methode aufgerufen.
void __fastcall Controller::onClick(TObject *Sender)
{
 Feld *feld = NULL;                  //Temporrer Speicher fr Felder
 feld = dynamic_cast<Feld*>(Sender); //Sender-Objekt wird zu Feld-Objekt gecastet

 if(this->isFinished())  //Falls das Spiel beendet wurde, wird abgebrochen
 {
   return;               //Abbrechen
 }

 if(this->getCurrentPlayer()->getType() == "Mensch"  //Falls der Spieler ein Mensch ist
 && (Sender == NULL || feld == NULL))                //Und kein Feld angeklickt wurde
 {
   return;                                           //Abbrechen
 }

 if(this->getCurrentPlayer()->getType() == "Mensch"
    && !feld->isEmpty()) //Falls der Spieler ein Mensch ist und das Feld belegt ist
 {
   return;                         //Wird die Methode abgebrochen, da nur leere
 }                                 //Felder angeklickt werden drfen


 //Sollte der aktuelle Spieler der PC sein, muss der Zug des PCs mit der Methode zugPC ausgefhrt werden
 if(this->getCurrentPlayer()->getType() == "Computer" && !this->isFinished())
 {
  Sleep(300);  //PC setzt nicht sofort, sondern wartet 300ms.

  this->zugPC(); //Der PC fhrt den Zug aus
 }
 //Der aktuelle Spieler ist kein PC, sondern ein Mensch. Klick wird ausgewertet
 else if(this->getCurrentPlayer()->getType() == "Mensch" && !this->isFinished())
 {
    if(feld->isEmpty())  //Man darf nur leere Felder anklicken
    {
     feld->setColor(this->getCurrentPlayer()->getColor());  //Angeklicktes Feld einfrben
     feld->setPlayer(this->getCurrentPlayer());             //Besitzer des Felds eintragen
    }
 }
 else
 {
     ShowMessage("Der Spielertyp des aktuellen Spielers ist undefiniert. Spielabbruch");
     this->setFinished(true);   //Spiel beenden, da Fehler auftrat.
 }

 this->searchWinner();    //Schauen, ob es einen Gewinner gibt / Unentschieden

 //Nur weitermachen, wenn das Spiel nicht beendet wurde.
 if(!this->isFinished())
 {
   //Spielzhler erhhen. Der aktuelle Spieler ist jetzt der nchste Spieler
   this->incrementCounter();

   //Status aktualisieren. Es wird jetzt der aktuelle (also der nchste) Spieler
   //im Status angezeigt.
   this->refreshStatus();

   //Diese Stelle ist sehr wichtig fr den Fall, dass ein Computerspieler an der Reihe ist.
   //Der PC lst nmlich kein onClick-Event aus, deshalb machen wir das hier manuell.
   if(this->getCurrentPlayer()->getType() == "Computer")
   {
     //Hier wird das onClick-Event manuell ausgelst (als ob der PC geklickt htte)
     this->onClick(NULL);
   }
 }
 else
 {
   this->incrementCounter(); //Wenn Spiel beendet wurde, muss der Spielzhler
                             //noch um 1 erhht werden, weil er sonst den
                             //beendenden Zug nicht enthlt.
 }
}

//Diese Methode wird aufgereufen, wenn der PC dran ist.
void Controller::zugPC()
{
 //Aktueller Spieler ist PC?
 if(this->getCurrentPlayer()->getType() != "Computer")
 {
   ShowMessage("Es ist ein schwerwiegender Fehler aufgetreten");
   this->setFinished(true);                                      //Spiel beenden
 }

 
 //getCurrentPlayer gibt einen Pointer auf ein Spieler-Objekt zurck.
 //Die Klasse Spieler kennt aber die Methoden von Computer nicht.
 //Deshalb mssen wir einen Type-Cast zu einem Computer*-Pointer durchfhren
 //Danach sind alle Methoden und Attribute des Objekts ansprechbar.
 Computer* spieler = dynamic_cast<Computer*>(this->getCurrentPlayer());

 //Je nach Schwierigkeit andere Methode ausfhren
 switch(spieler->getSchwierigkeit())
 {
    case 1:  this->pcEasy();                    //PC spielt mit pcEasy()
             break;
    case 2:  this->pcDifficult();               //PC spielt mit pcDifficult()
             break;
    default: ShowMessage("Undefinierte Schwierigkeitsstufe");
             break;
 }
}

/*
   Die einfache Spielmethode setzt an einer zuflligen Stelle im Spielfeld
   einen Stein, falls das Feld frei ist.
*/
void Controller::pcEasy()
{
 Feld *feld = NULL;  //Temporrer Speicher fr Feld
 int x = 0, y = 0;   //Koordinaten des PC-Zugs

 randomize();        //Zufallsgenerator starten

 do
 {
   x =  random(CONST__anzX);                //zufllige x-Koordinate
   y =  random(CONST__anzY);                //zufllige y-Koordinate
   if(this->model->isValidCoord(x,y))       //Prfen, ob Koordinate erlaubt ist
   {
     feld = this->model->getFeld(x,y);      //Das Feld (x|y) holen
   }
 }
 //solange das Spiel nicht unentschieden, beendet oder das Feld voll ist.
 while(!this->isUnentschieden() && feld != NULL && !feld->isEmpty() && !this->isFinished());


 feld->setColor(this->getCurrentPlayer()->getColor()); //Dem gefundenen Feld die Farbe zuweisen
 feld->setPlayer(this->getCurrentPlayer());            //Dem gefundenen Feld den Besitzer zuweisen
}

/*
   Die schwere Spielmethode versucht, eigene Kombinationen zu erstellen und zu
   vervollstndigen.
   Auerdem findet diese Methode die "Gewinnversuche" des Gegners und versucht,
   diese zu verhindern.
*/
void Controller::pcDifficult()
{
 Feld *feld = NULL;  //Temporrer Speicher fr Feld

 //Die ersten beiden Zge sollen nicht mit der KI gemacht werden,
 //weil der PC sonst am Anfang immer den gleichen Zug macht, wenn der Mensch
 //das auch macht.
 if(this->getCounter() <= 1)
 {
  this->pcEasy();
  return;         //Abbruch, da Zug gemacht wurde.
 }

 //Als erstes wird versucht, eigene 3er-Kombinationen zu erzeugen
 feld = this->findCombinations(this->getCurrentPlayer(), 2);

 if(feld == NULL)
 { //Dann wird versucht, gegnerische 2er zu blockieren, so dass keine 3er entstehen
  feld = this->findCombinations(this->getNextPlayer(), 2);
 }

 if(feld == NULL)
 { //Andernfalls werden aus eigenen 1ern 2er gemacht
  feld = this->findCombinations(this->getCurrentPlayer(), 1);
 }

 if(feld == NULL)
 {  //Sonst gegnerische 1er blockieren, so dass keine 2er entstehen
  feld = this->findCombinations(this->getNextPlayer(), 1);
 }

 if(feld == NULL)
 { //Sollte nichts geklappt haben, wird die Zufallsmethode versucht
   this->pcEasy();
 }
 else
 { //Wenn die KI zugeschlagen hat, wird das Feld belegt
   feld->setColor(this->getCurrentPlayer()->getColor());
   feld->setPlayer(this->getCurrentPlayer());
 }
}

/*
  Diese Methode durchsucht das Spielfeld nach Kombinationen mit der Lnge "length"
  Anschlieend gibt sie das Feld, das "vor" der Kombination kommt und leer ist,
  zurck.
*/
Feld* Controller::findCombinations(Spieler* player, unsigned int length)
{

 for(int y = 0; y < CONST__anzY; y++)
 {
   for(int x = 0; x < CONST__anzX; x++)
   {
     if(this->model->isValidCoord(x,y) &&            //Koordinate gltig?
        this->model->getFeld(x,y)->isEmpty()         //Feld leer?
        )
     {
       if(this->isCombination(player, x, y+1, 0, 1, length)     //Senkrecht
       || this->isCombination(player, x+1, y, 1, 0, length)     //Waagrecht
       || this->isCombination(player, x, y-1, 0, -1, length)    //Senkrecht
       || this->isCombination(player, x-1, y, -1, 0, length)    //Waagrecht
       || this->isCombination(player, x+1, y+1, 1, 1, length)   //Diagonal
       || this->isCombination(player, x+1, y-1, 1, -1, length)  //Diagonal
       || this->isCombination(player, x-1, y+1, -1, 1, length)  //Diagonal
       || this->isCombination(player, x-1, y-1, -1, -1, length) //Diagonal
       )
       {
          return this->model->getFeld(x,y);             //Feld zurckgeben
       }
     }
   }
  }
  return NULL;                                         //Kein Feld gefunden
}

/**
  Hier wird geprft, ob einer der Spieler das Spiel gewonnen hat oder ob das
  Spiel unentschieden ist.
*/
void Controller::searchWinner()
{
Spieler *winner = NULL;             //Temporrer Speicher fr den Gewinner-Spieler
winner = this->checkField(); //Das Spielfeld nach dem Gewinner durchsuchen
if(winner != NULL)        //Wenn winner != NULL ist, gibt es einen Gewinner.
{
  this->gui->setStatusMessage(winner->getName()+" hat gewonnen!");   //Gewinner ausgeben
  this->gui->setSpielerfarbeVorschau(winner->getColor());            //Farbe des Gewinners anzeigen
  this->setFinished(true);                                           //Spiel beenden
}
else
{
  //Auf unentschieden prfen
  if(this->isUnentschieden())                           //Prfen, ob unentschieden ist
  {
   this->gui->setStatusMessage("Untentschieden");       //Unentschieden ausgeben
   this->gui->resetSpielerfarbeVorschau();              //Farbe zurcksetzen
   this->setFinished(true);                             //Spiel beenden
  }
}
}

/**
   Gibt die ID (index im spieler-Array) des aktuellen Spielers zurck
*/
int Controller::getCurrentPlayerID()
{
  return  this->counter % 2 == 0 ? 0 : 1;
}

/*
   gibt den aktuellen Spieler zurck (Pointer auf Spieler-Objekt)
*/
Spieler* Controller::getCurrentPlayer()
{
  //Das Spieler-Objekt mit dem ermittelten Index wird returned
  return this->spieler[this->getCurrentPlayerID()];
}

/**
   Gibt die ID (index im spieler-Array) des nchsten Spielers zurck
*/
int Controller::getNextPlayerID()
{
  return (this->getCurrentPlayerID() == 1) ? 0 : 1;   //"toggle"
}

/*
   gibt den nchsten Spieler zurck (Pointer auf Spieler-Objekt)
*/
Spieler* Controller::getNextPlayer()
{
  //Das Spieler-Objekt mit dem ermittelten Index wird returned
  return this->spieler[this->getNextPlayerID()];
}

/*
   Prft, ob alle Felder belegt sind und das Spiel somit beendet ist.
*/
bool Controller::isUnentschieden()
{
   int volle_felder = 0;  //Wieviele Felder sind bereits belegt?

   //In dieser verschachtelten for-Schleife wird ermittelt, wieviele Felder belegt sind.
   for(int y = 0; y < CONST__anzY; y++)
   {
     for(int x = 0; x < CONST__anzX; x++)
     {
       if(this->model->isValidCoord(x,y) &&     //Gltige Koordinate &&
         !this->model->getFeld(x,y)->isEmpty()  //Feld belegt?
         )
       {
         volle_felder++;    //Ein belegtes Feld mehr
       }
     }
   }

 //Entspricht die Anzahl der vollen Felder der Anzahl der vorhandenen Spielfelder?
 return (volle_felder >= (CONST__anzX * CONST__anzY));
}

//Diese Methode leitet den Aufruf direkt an die GUI weiter (Delegation)
void __fastcall Controller::adjust(TObject *Sender)
{
  this->gui->placeElements();
}

/*
  isCombination berprft, ob es ab der Stelle (startX|startY) eine
  Feldkombination des Spielers player gibt.

  Mit dirX und dirY kann man steuern, ob waagrechte, senkrechte oder diagonale
  Kombinationen gesucht werden sollen.
  length gibt die Mindestlnge der Kombinationen an.
*/
bool Controller::isCombination(Spieler* player, int startX, int startY, int dirX, int dirY, unsigned int length)
{
   //Teste, ob Startkoordinaten gltig sind
   if(!this->model->isValidCoord(startX, startY))
   {
       return false; //Ungltige Koordinaten
   }

   //Teste, ob Endkoordinaten gltig sind
   if(!this->model->isValidCoord(startX + (dirX*(length-1)), startY + (dirY*(length-1))))
   {
      return false; //Ungltige Koordinaten
   }

   //jetzt wird eine Schleife length mal wiederholt.
   for(unsigned int i = 0; i < length; i++)
   {
     //Es wird geprft, ob das Feld NICHT vom Spieler ist.
     if(this->model->getFeld(startX + (dirX*i), startY + (dirY*i))->getPlayer() != player)
     {
       return false;    //Wenn es nicht vom Spieler ist, wird false zurckgegeben,
                        //da dann keine Kombination existiert.
     }
   }
   return true;         //Eine Kombination existiert!
}

/*
    berprft, ob es im Spielfeld einen Gewinner gibt, der length
    Felder in einer Kombination hat.
 */
Spieler* Controller::checkField()
{
  unsigned int length = 3;    //TicTacToe erfordert 3er-Kombinationen

  Spieler* player = NULL;     //Pointer auf spieler-Objekt

  //Spielfeld durchlaufen
  for(unsigned int y = 0; y < CONST__anzY; y++)
  {
      for(unsigned int x = 0; x < CONST__anzX; x++)
      {
           if(this->model->isValidCoord(x,y))      //Ist die Koordinate gltig?
           {

            player =  this->model->getFeld(x, y)->getPlayer();    //Besitzer des Felds holen
            if(player != NULL)                                    //Wenn es einen Besitzer gibt
            {
             if(   this->isCombination(player, x, y, 0, 1, length)   //Senkrecht
                || this->isCombination(player, x, y, 1, 0, length)   //Waagrecht
                || this->isCombination(player, x, y, 1, 1, length)   //Diagonal
                || this->isCombination(player, x, y, 1, -1, length)  //Diagonal
                || this->isCombination(player, x, y, -1, 1, length)  //Diagonal
                || this->isCombination(player, x, y, -1, -1, length) //Diagonal
                )
             {
               return player;                    //Gewinner zurckgeben
             }
            }
           }
      }
  }
  return NULL;                                   //Kein Gewinner
}

/*
  Gibt entweder Spieler 1 oder Spieler 2 zurck.
*/
Spieler* Controller::getSpieler(int nr)
{
  if(nr != 1 && nr != 2)         // nr muss 1 oder 2 sein, weil es nur 2 Spieler gibt
  { throw "Dieser Spieler existiert nicht"; }

  return this->spieler[nr-1];   //Array beginnt bei 0 --> -1
}

/*
   Gibt Pointer auf Model-Objekt zurck
*/
SpielfeldModel* Controller::getModel()
{
  return this->model;
}

/*
   Erhht den Counter, der die Spielzge zhlt.
*/
void Controller::incrementCounter()
{
  this->counter++;
  this->gui->showSpielzuege(this->counter);  //Neue Anzahl anzeigen
}

/*
  Verringert den Counter, der die Spielzge zhlt
*/
void Controller::decrementCounter()
{
  this->counter--;
  this->gui->showSpielzuege(this->counter);   //Neue Anzahl anzeigen
}

/*
  Counterstand zurckgeben
*/
int Controller::getCounter()
{
  return this->counter;
}

/**
   Counter zurckstetzen
*/
void Controller::resetCounter()
{
  this->counter = 0;
  this->gui->showSpielzuege(this->counter);   //Neue Anzahl anzeigen
}

/*
   Controller zurcksetzten und neues Spiel starten
*/
void Controller::reset()
{
  this->resetCounter();       //Counter zurcksetzen
  this->model->reset();       //Spielfeld leeren
  this->refreshStatus();      //Status aktualisieren
  this->gui->RefreshField();  //Feld neu anzeigen
  this->resetTimer();
  this->setFinished(false);   //Spiel starten
}

/*
   Spiel beenden / starten
*/
void Controller::setFinished(bool finished)
{
  this->finished = finished;
}

/**
  Wurde das Spiel beendet?
*/
bool Controller::isFinished()
{
  return this->finished;
}

/*
   Status aktualisieren
   Status enthlt den aktuellen Spielernamen und seine Farbe
*/
void Controller::refreshStatus()
{
  //Spielername anzeigen
  this->gui->setStatusMessage("Spieler '" + this->getCurrentPlayer()->getName() + "' ist dran!");

  //Farbe anzeigen
  this->gui->setSpielerfarbeVorschau(this->getCurrentPlayer()->getColor());

  //nderungen von der GUI verarbeiten lassen
  Application->ProcessMessages();
}

/*
   Mit dieser Methode kann man den Typ (Computer / Mensch)
   eines Spielers ndern.
   Die ID entspricht hier nicht der ID im array, sondern ist 1 hher.
*/
void Controller::setPlayerType(int id, AnsiString type)
{
  //Spieler 1 oder SPieler 2 NICHT?
  if(id != 1 && id != 2)
  {
    throw "Dieser Spieler existiert nicht!";
  }

  id--;  //ID entspricht jetzt Array-Index von this->spieler

  if(type == "Mensch")
  {
    this->spieler[id] = new Mensch;   //MEnsch-Objekt hineinschreiben
  }
  else if(type == "Computer")
  {
     this->spieler[id] = new Computer;   //Computer-Objekt hineinschreiben
  }
  else
  {
    throw "Dieser Spielertyp existiert nicht";
  }
}

//Gibt einen Zeiger auf die GUI (Form1) zurck
TForm1* Controller::getGUI()
{
  return this->gui;
}

//Diese Methode wird aufgerufen, wenn der Timer aktiv wird.
void Controller::refreshTimer()
{
    if(!this->isFinished()) //Zeit nur zhlen, wenn Spiel nicht beendet ist
    {
       this->timer_sec++;         //Bei jedem Aufruf soll der Sekundenzhler erhht werden.
       if(this->timer_sec >= 60)  //Prfen, ob eine Minute vorbei ist
       {
         this->timer_sec = 0;     //Wenn ja, Sekunden auf 0 setzen
         this->timer_min++;       //und Minuten erhhen
       }
    }

    this->gui->showTimerData(this->timer_min, this->timer_sec); //Die GUI soll die Daten jetzt ausgeben
}

/*
    Setzt die Spieldauer auf 0:00 min zurck
 */
void Controller::resetTimer()
{
  this->timer_sec = 0;      //0 Sekunden
  this->timer_min = 0;      //0 Minuten
  this->gui->showTimerData(this->timer_min, this->timer_sec);  //Anzeige neu beschreiben lassen
}