/*
   Der Controller fr die Optionen schafft eine Verbindung zwischen
   GUI und Model.
   Er gibt die Daten zur Speicherung weiter und gibt die ausgelesenen Daten
   an die GUI zur Anzeige.
*/
class ControllerOptions
{
  private:
    ModelOptions* model;               //Pointer auf Model
    TForm2* gui;                       //Pointer auf GUI
    Controller* main_controller;       //Pointer auf Hauptcontroller
    bool restart_on_save;              //Nach Speichern neu starten?

  public:
    ControllerOptions(TForm2* pgui, Controller* maincontroller)
    {
      this->restart_on_save = false;                //Beim Speichern das Spiel neustarten?
      this->model = new ModelOptions;               //Erstellen des Models
      this->gui   = pgui;                           //GUI zuweisen
      this->main_controller = maincontroller;       //Hauptcontroller zuweisen
    }

    void storeSettings();                           //Einstellungen speichern
    void importSettingsFromFile(AnsiString);        //INI-Datei importieren

    void ok();                                      //Schlieen der Optionen
    void reset();                                   //Zurcksetzen der Optionen
    void exportSettings();                          //Exportieren
    void importSettings();                          //Importieren
    void init();                                    //Initialisieren
    void setRestartOnSave(bool);                    //Beim Speichern neu starten?
    void changeTypeOfPlayer(int nr);                //Spielertyp ndern
};

//Wird beim Klick auf OK oder beim Schlieen aufgerufen.
void ControllerOptions::ok()
{
  this->storeSettings();             //Einstellungen bernehmen
  this->gui->Visible = false;        //Optionsmen verstecken
}

//Hier kann eingestellt werden, ob das Spiel nach dem Speichern neu gestartet
//werden muss
void ControllerOptions::setRestartOnSave(bool r)
{
  this->restart_on_save = r;
}

/*
  Wird aufgerufen, wenn das Formular erzeugt wird
  (Also beim Programmstart)

  Hier werden Werte bernommen und Einstellungen aus einer INI-Importiert,
  falls eine existiert.
*/
void ControllerOptions::init()
{
   //Die aktuellen Farben mssen in der GUI angezeigt werden (Optionsmen)
   this->gui->farbe_spieler1->Color =  this->main_controller->getSpieler(1)->getColor();
   this->gui->farbe_spieler2->Color =  this->main_controller->getSpieler(2)->getColor();
   this->gui->farbe_hintergrund->Color =  this->main_controller->getModel()->getBackgroundColor();
   this->gui->farbe_leeresfeld->Color = this->main_controller->getModel()->getEmptyFieldColor();

   //Die aktuellen Spielernamen anzeigen
   this->gui->spieler1_name->Text    = this->main_controller->getSpieler(1)->getName();
   this->gui->spieler2_name->Text    = this->main_controller->getSpieler(2)->getName();

  //Prfen, ob Pfad zur INI existiert.
  if(this->model->getINIPath() != "")
  {
     //Importieren der Einstellungen aus der in pathinfo.ttt angegebenen Datei
     this->importSettingsFromFile(this->model->getINIPath());

     //Spiel zurcksetzen
     this->main_controller->reset();
  }
}

//Einstellungen in eine INI-Datei exportieren
void ControllerOptions::exportSettings()
{
   //Es soll automatisch als erstes der Ordner angezeigt werden, wo das letzte Mal
   //gespeichert wurde
   this->gui->SaveDialog1->InitialDir = ExtractFilePath(this->model->getINIPath());
   //Den Dialog zur Auswahl der INI-Datei anzeigen
   this->gui->SaveDialog1->Execute();

   //Prfen, ob eine Datei gewhlt wurde
   if(this->gui->SaveDialog1->FileName == "")
   {
       ShowMessage("Keine Datei gewhlt");
       return;                          //Abbrechen
   }

   //Prfen, ob gewhlte Datei existiert
   if(FileExists(this->gui->SaveDialog1->FileName))
   {
      //Fragen, ob berschrieben werden darf
      if(Application->MessageBox((this->gui->SaveDialog1->FileName +(AnsiString)" ist schon vorhanden").c_str(), "Sicher?", MB_YESNO) == IDNO)
      {
        return;                      //Abbrechen
      }
   }

   //Ein neues Zugriffsobjekt auf die INI-Datei wird erstellt
   TIniFile* ini = new TIniFile(this->gui->SaveDialog1->FileName);

   this->model->setINIPath(this->gui->SaveDialog1->FileName);

   //Spielernamen in Einstellungsdatei schreiben
   ini->WriteString("name", "spieler1", this->gui->spieler1_name->Text);
   ini->WriteString("name", "spieler2", this->gui->spieler2_name->Text);

   //Alle Farben als (int) kodiert in Einstellungsdatei schreiben
   ini->WriteInteger("farben", "hintergrund", (unsigned int)this->gui->farbe_hintergrund->Color);
   ini->WriteInteger("farben", "leer",        (unsigned int)this->gui->farbe_leeresfeld->Color);
   ini->WriteInteger("farben", "spieler1",    (unsigned int)this->gui->farbe_spieler1->Color);
   ini->WriteInteger("farben", "spieler2",    (unsigned int)this->gui->farbe_spieler2->Color);

   //Spielertypen in Einstellungsdatei schreiben
   ini->WriteInteger("spieler", "spieler1",    this->gui->typ_spieler1->ItemIndex);
   ini->WriteInteger("spieler", "spieler2",    this->gui->typ_spieler2->ItemIndex);

   //PC-Schwierigkeit schreiben
   ini->WriteInteger("schwierigkeit", "pc1",  this->gui->dif_pc1->ItemIndex);
   ini->WriteInteger("schwierigkeit", "pc2",  this->gui->dif_pc2->ItemIndex);

   //Erfolgsmeldung anzeigen
   ShowMessage("Die Einstellungen wurden unter "+this->gui->SaveDialog1->FileName+" abgespeichert");

   //Speicher wieder freigeben
   delete ini;
}

//Importieren der Einstellungen
void  ControllerOptions::importSettings()
{
   //Es soll automatisch als erstes der Ordner angezeigt werden, wo das letzte Mal
   //gespeichert wurde
   this->gui->OpenDialog1->InitialDir = ExtractFilePath(this->model->getINIPath());

   //Datei ffnen-Dialog anzeigen
   this->gui->OpenDialog1->Execute();

   //Wurde eine Datei gewhlt?
   if(this->gui->OpenDialog1->FileName == "")
   {
     ShowMessage("Es wurde keine Datei gewhlt");
     return;                  //Abbruch
   }

   //Diese Methode importiert dann die Einstellungen aus der gewhlten Datei
   this->importSettingsFromFile(this->gui->OpenDialog1->FileName);
}

//Alles auf Standarteinstellungen setzen
void ControllerOptions::reset()
{
 //Zuerst fragen, ob die Einstellungen zurckgesetzt werden sollen
  if(Application->MessageBox("Einstellungen zurcksetzen?", "Sicher?", MB_YESNO) == IDYES)
  {
    //Spielernamen setzen
    this->gui->spieler1_name->Text = CONST__default_spieler1_name;
    this->gui->spieler2_name->Text = CONST__default_spieler2_name;

    //Die Spielertypen-Auswahl setzen
    this->gui->typ_spieler1->ItemIndex = 0;
    this->gui->typ_spieler2->ItemIndex = 1;

    //Die Schwierigkeit einstellen
    this->gui->dif_pc1->ItemIndex     = 0;
    this->gui->dif_pc2->ItemIndex     = 1;

    //Spielerfarben setzen
    this->gui->farbe_spieler1->Color = CONST__default_spieler1_color;
    this->gui->farbe_spieler2->Color = CONST__default_spieler2_color;

    //Andere Farben setzen
    this->gui->farbe_hintergrund->Color = CONST__default_background_color;
    this->gui->farbe_leeresfeld->Color = CONST__default_emptyField_color;
  }
}

//Speichern der Einstellungen
void ControllerOptions::storeSettings()
{
  //Wenn die Spielereinstellungen gendert wurden
  //Dann muss das Spiel neugestartet werden.
  //Davor werden die Spieler so erzeugt, wie eingestellt wurde.
  if(this->restart_on_save)
  {

     //Typ des ersten Spielers setzen
     this->main_controller->setPlayerType(1, this->gui->typ_spieler1->ItemIndex == 0 ? "Mensch" : "Computer");

     //Typ des zweiten Spielers setzen
     this->main_controller->setPlayerType(2, this->gui->typ_spieler2->ItemIndex == 0 ? "Mensch" : "Computer");

     //Falls der erste Spieler ein PC ist, wird noch die Schwierigkeit eingestellt
     if(this->gui->typ_spieler1->ItemIndex == 1)
     {
       //der Cast macht aus Spieler* Computer*
       dynamic_cast<Computer*>(this->main_controller->getSpieler(1))
       ->setSchwierigkeit(this->gui->dif_pc1->ItemIndex == 0 ? 1 : 2);
     }

     //Falls der zweite Spieler ein PC ist, wird noch die Schwierigkeit eingestellt
     if(this->gui->typ_spieler2->ItemIndex == 1)
     {
       //der Cast macht aus Spieler* Computer*
       dynamic_cast<Computer*>(this->main_controller->getSpieler(2))
       ->setSchwierigkeit(this->gui->dif_pc2->ItemIndex == 0 ? 1 : 2);
     }

     //Spiel neustarten
     this->main_controller->reset();
  }

  //Die Spielernamen setzen
  this->main_controller->getSpieler(1)->setName(this->gui->spieler1_name->Text);
  this->main_controller->getSpieler(2)->setName(this->gui->spieler2_name->Text);

  //Die Spielerfarben setzen
  this->main_controller->getSpieler(1)->setColor(this->gui->farbe_spieler1->Color);
  this->main_controller->getSpieler(2)->setColor(this->gui->farbe_spieler2->Color);

  //Die anderen Farben setzen
  this->main_controller->getModel()->setBackgroundColor(this->gui->farbe_hintergrund->Color);
  this->main_controller->getModel()->setEmptyFieldColor(this->gui->farbe_leeresfeld->Color);

  this->main_controller->refreshStatus(); //Status aktualisieren
  this->main_controller->getGUI()->RefreshField();    //Spielfeld neu aufbauen
}

//Anhand einer Datei, deren Name bergeben wird, importiert diese Methode
//alle Einstellungen aus dieser Datei und weit sie dem Spiel zu.
void ControllerOptions::importSettingsFromFile(AnsiString filename)
{
   //Wenn die Datei nicht existiert, wird einfach abgebrochen.
   if(!FileExists(filename))
   {
     return;
   }

    this->model->setINIPath(filename);

    //Neues Objekt zum INI-Zugriff
    TIniFile* ini = new TIniFile(filename);

    //Read*() erwartet immer 3 Parameter:
    //-Die [Sektion] in der INI-Datei
    //-Den Namen des Attributs
    //-Den Standartwert, falls das Attribut in der INI nicht gefunden wird

    //Hier werden Farben zugewiesen. (der letzte Paramter ist der "aktuelle" wert, der momentan eingestellt ist)
    this->gui->farbe_hintergrund->Color = (TColor)ini->ReadInteger("farben", "hintergrund", this->gui->farbe_hintergrund->Color);
    this->gui->farbe_leeresfeld->Color = (TColor)ini->ReadInteger("farben", "leer",   this->gui->farbe_leeresfeld->Color);
    this->gui->farbe_spieler1->Color = (TColor)ini->ReadInteger("farben", "spieler1", this->gui->farbe_spieler1->Color);
    this->gui->farbe_spieler2->Color = (TColor)ini->ReadInteger("farben", "spieler2", this->gui->farbe_spieler2->Color);

    //Hier werden die Spielertypen zugewiesen
    this->gui->typ_spieler1->ItemIndex = ini->ReadInteger("spieler", "spieler1", this->gui->typ_spieler1->ItemIndex);
    this->gui->typ_spieler2->ItemIndex = ini->ReadInteger("spieler", "spieler2", this->gui->typ_spieler2->ItemIndex);

    //Die Schwierigkeiten der Bots
    this->gui->dif_pc1->ItemIndex = ini->ReadInteger("schwierigkeit", "pc1", this->gui->dif_pc1->ItemIndex);
    this->gui->dif_pc2->ItemIndex = ini->ReadInteger("schwierigkeit", "pc2", this->gui->dif_pc2->ItemIndex);

    //Die Spielernamen
    this->gui->spieler1_name->Text = ini->ReadString("name", "spieler1", CONST__default_spieler1_name);
    this->gui->spieler2_name->Text = ini->ReadString("name", "spieler2", CONST__default_spieler2_name);

    //Die Einstellungen speichern (Dem Spiel/den Objekten zuweisen)
    this->storeSettings();

    //Speicher freigeben
    delete ini;
}

/*
   ndert den Spielertyp (speichert nichts, sondern zeigt nur die richtigen Werte an)
*/
void ControllerOptions::changeTypeOfPlayer(int nr)
{

  TRadioGroup* rgrp_type = NULL;                //Radiogroup Typ
  TEdit* edt_name = NULL;                       //Textfeld Name
  TRadioGroup* rgrp_diff = NULL;                //Radiogroup Schwierigkeit

  //Um den richtigen Spieler zu erwischen, werden jetzt ein paar Werte zugewiesen
  if(nr == 1)
  {
      rgrp_type = this->gui->typ_spieler1;
      edt_name = this->gui->spieler1_name;
      rgrp_diff = this->gui->dif_pc1;
  }
  else
  {
      rgrp_type = this->gui->typ_spieler2;
      edt_name = this->gui->spieler2_name;
      rgrp_diff = this->gui->dif_pc2;
  }

  if(rgrp_type->ItemIndex == 0) //Mensch
  {
    edt_name->Text = "Mensch";
    rgrp_diff->Enabled = false;     //Mensch hat keine Schwierigkeitsstufe
  }
  else                                    //PC
  {
    edt_name->Text = "Computer";
    rgrp_diff->Enabled = true;        //PC-Schwierigkeit wird einstellbar
  }

  this->setRestartOnSave(true);//Das Spiel muss neugestartet werden,
                                     //Wenn der Spielertyp gendert wurde
}