/*
  Das SpielfeldModel ist eine Klasse, die zur Speicherung des Spielfelds dient.
  Sie hat alle Felder in einem Array und bietet Methoden zum Ansprechen dieser
  Felder.
*/

class SpielfeldModel
{
  private:
    Feld* array[CONST__anzX][CONST__anzY];            //Array mit allen Feldern
    TColor emptyFieldColor;                           //Farbe eines leeren Felds
    TColor backgroundColor;                           //Hintergrundfarbe

  public:
   //Konstruktor
   SpielfeldModel(
          TComponent* Owner,
          TWinControl* parent,
          unsigned int width,
          unsigned int height,
          unsigned int top,
          unsigned int left)
   {

   //Zuweisung der Standartfarben
   this->emptyFieldColor = CONST__default_emptyField_color;
   this->backgroundColor = CONST__default_background_color;

   //Das Array durchlaufen
   for(int y = 0; y < CONST__anzY; y++)
   {
      for(int x = 0; x < CONST__anzX; x++)
      {
        //Jedem Arrayelement ein Feld zuweisen.
        this->array[x][y] = new Feld("x" + IntToStr(x) + "_y" + IntToStr(y), //Feldname
                                     Owner, parent,        //Besitzer und Vater
                                     width, height,        //Mae des Felds
                                     y*top, x*left,        //Koordinaten des Felds
                                     this->emptyFieldColor //Farbe des Felds
                                     );
      }
   }

   //Die Hintergrundfarbe des Elternobjekts wird gesetzt.
   dynamic_cast<TPanel*>(parent)->Color = this->backgroundColor;
   }

   /*
      Liefert ein Feld des Spielfelds
   */
   Feld* getFeld(int x, int y)
   {
     //Prfen, ob gltige Koordinaten bergeben wurden
     if(this->isValidCoord(x,y))
     {
      return this->array[x][y];     //Zurckgeben
     }
     else
     {
       throw "Dieses Feld besitzt ungueltige Koordinaten";
     }
   }

   //Farbe der leeren Felder setzen
   void setEmptyFieldColor(TColor cl)
   {
     this->emptyFieldColor = cl;
   }

   //Farbe der leeren Felder holen
   TColor getEmptyFieldColor()
   {
     return this->emptyFieldColor;
   }

   //Hintergrundfarbe setzen
   void setBackgroundColor(TColor cl)
   {
      this->backgroundColor = cl;
   }

   //Hintergrundfarbe holen
   TColor getBackgroundColor()
   {
      return this->backgroundColor;
   }

   //Prfen, ob eine Koordinate gltig ist
   bool isValidCoord(int x, int y)
   {
     //Prft die x- und y-Koordinate seperat
     return (this->isValidCoordX(x) && this->isValidCoordY(y));
   }

   //x-Koordinate erlaubt?
   bool isValidCoordX(int x)
   {
      return (x >= 0 && x < CONST__anzX);  
   }

   //y-Koordinate erlaubt?
   bool isValidCoordY(int y)
   {
      return (y >= 0 && y < CONST__anzY);
   }

   //Spielfeld zurcksetzen auf Anfangszustand
   void reset()
   {
    //Spielfeld durchlaufen
    for(int y = 0; y < CONST__anzY; y++)
    {
      for(int x = 0; x < CONST__anzX; x++)
      {
        //Feld leeren
        this->getFeld(x,y)->setEmpty();
      }
    }
   }
};
